package com.example.springip.lab3.controller;

import com.example.springip.lab3.dto.CompanyDto;
import com.example.springip.lab3.dto.PositionDto;
import com.example.springip.lab3.service.PositionService;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/position")
public class PositionController {
    private final PositionService positionService;

    public PositionController(PositionService positionService) {
        this.positionService = positionService;
    }

    @GetMapping("/{id}")
    public PositionDto getCompany(@PathVariable long id) {
        return new PositionDto(positionService.findPosition(id));
    }

    @GetMapping
    public List<PositionDto> getAllCompanies() {
        return positionService.findAllPositions().stream()
                .map(PositionDto::new)
                .toList();
    }

    @PostMapping
    public PositionDto create(@RequestParam("name") String name) {
        return new PositionDto(positionService.addPosition(name));
    }

    @PutMapping("/{id}")
    public PositionDto update(@PathVariable Long id,
                             @RequestParam("name") String name) {
        return new PositionDto(positionService.updatePosition(id, name));
    }

    @DeleteMapping("/{id}")
    public PositionDto delete(@PathVariable Long id) {
        return new PositionDto(positionService.deletePosition(id));
    }
}
